@@ -1,10 +1,9 @@
 """Tools for calculating forecast errors."""
 import warnings
 import numpy as np
-import pandas as pd
 
 
-def smape(actual, forecast):
+def symmetric_mean_absolute_percentage_error(actual, forecast):
     """Expect two, 2-D numpy arrays of forecast_length * n series.
     Allows NaN in actuals, and corresponding NaN in forecast, but not unmatched NaN in forecast
     Also doesn't like zeroes in either forecast or actual - results in poor error value even if forecast is accurate
@@ -27,7 +26,7 @@ def smape(actual, forecast):
     return smape
 
 
-def mae(A, F):
+def mean_absolute_error(A, F):
     """Expects two, 2-D numpy arrays of forecast_length * n series.
 
     Returns a 1-D array of results in len n series
@@ -43,6 +42,22 @@ def mae(A, F):
     return mae_result
 
 
+def median_absolute_error(A, F):
+    """Expects two, 2-D numpy arrays of forecast_length * n series.
+
+    Returns a 1-D array of results in len n series
+
+    Args:
+        A (numpy.array): known true values
+        F (numpy.array): predicted values
+    """
+    mae_result = abs(A - F)
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", category=RuntimeWarning)
+        mae_result = np.nanmedian(mae_result, axis=0)
+    return mae_result
+
+
 def pinball_loss(A, F, quantile):
     """Bigger is bad-er."""
     with warnings.catch_warnings():
@@ -52,8 +67,15 @@ def pinball_loss(A, F, quantile):
     return result
 
 
-def SPL(A, F, df_train, quantile):
-    """Scaled pinball loss."""
+def scaled_pinball_loss(A, F, df_train, quantile):
+    """Scaled pinball loss.
+
+    Args:
+        A (np.array): actual values
+        F (np.array): forecast values
+        df_train (np.array): values of historic data for scaling
+        quantile (float): which bound of upper/lower forecast this is
+    """
     # scaler = df_train.tail(1000).diff().abs().mean(axis=0)
     # scaler = np.abs(np.diff(df_train[-1000:], axis=0)).mean(axis=0)
     with warnings.catch_warnings():
@@ -72,7 +94,7 @@ def SPL(A, F, df_train, quantile):
     return pl / scaler
 
 
-def rmse(actual, forecast):
+def root_mean_square_error(actual, forecast):
     """Expects two, 2-D numpy arrays of forecast_length * n series.
 
     Returns a 1-D array of results in len n series
@@ -96,15 +118,12 @@ def containment(lower_forecast, upper_forecast, actual):
         actual (numpy.array): known true values
         forecast (numpy.array): predicted values
     """
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore", category=RuntimeWarning)
-        result = (
-            np.count_nonzero(
-                (upper_forecast >= actual) & (lower_forecast <= actual), axis=0
-            )
-            / actual.shape[0]
+    return (
+        np.count_nonzero(
+            (upper_forecast >= actual) & (lower_forecast <= actual), axis=0
         )
-    return result
+        / actual.shape[0]
+    )
 
 
 def contour(A, F):
@@ -133,108 +152,33 @@ def contour(A, F):
     return contour_result
 
 
-class EvalObject(object):
-    """Object to contain all the failures!."""
-
-    def __init__(
-        self,
-        model_name: str = 'Uninitiated',
-        per_series_metrics=np.nan,
-        per_timestamp=np.nan,
-        avg_metrics=np.nan,
-        avg_metrics_weighted=np.nan,
-        full_mae_error=None,
-    ):
-        self.model_name = model_name
-        self.per_series_metrics = per_series_metrics
-        self.per_timestamp = per_timestamp
-        self.avg_metrics = avg_metrics
-        self.avg_metrics_weighted = avg_metrics_weighted
-        self.full_mae_error = full_mae_error
+def rmse(ae):
+    """Accepting abs error already calculated"""
+    return np.sqrt(np.nanmean((ae ** 2), axis=0))
 
 
-def PredictionEval(
-    PredictionObject,
-    actual,
-    series_weights: dict,
-    df_train=None,
-    per_timestamp_errors: bool = False,
-    full_mae_error: bool = False,
-    dist_n: int = None,
-):
-    """Evalute prediction against test actual.
+def mae(ae):
+    """Accepting abs error already calculated"""
+    return np.nanmean(ae, axis=0)
 
-    This fails with pd.NA values supplied.
 
-    Args:
-        PredictionObject (autots.PredictionObject): Prediction from AutoTS model object
-        actual (pd.DataFrame): dataframe of actual values of (forecast length * n series)
-        series_weights (dict): key = column/series_id, value = weight
-        df_train (pd.DataFrame): historical values of series, used for setting scaler for SPL
-            if None, actuals are used instead. Suboptimal.
-        per_timestamp (bool): whether to calculate and return per timestamp direction errors
-        dist_n (int): if not None, calculates two part rmse on head(n) and tail(remainder) of forecast.
-        full_mae_error (bool): if True, return all MAE values for all series and timestamps
-    """
-    A = np.array(actual)
-    F = np.array(PredictionObject.forecast)
-    lower_forecast = np.array(PredictionObject.lower_forecast)
-    upper_forecast = np.array(PredictionObject.upper_forecast)
-    if df_train is None:
-        df_train = actual
-    df_train = np.array(df_train)
-    # make sure the series_weights are passed correctly to metrics
-    if len(series_weights) != F.shape[1]:
-        series_weights = {
-            col: series_weights[col] for col in PredictionObject.forecast.columns
-        }
+def medae(ae):
+    """Accepting abs error already calculated"""
+    return np.nanmedian(ae, axis=0)
 
-    errors = EvalObject(model_name=PredictionObject.model_name)
 
-    per_series = pd.DataFrame(
-        {
-            'smape': smape(A, F),
-            'mae': mae(A, F),
-            'rmse': rmse(A, F),
-            'containment': containment(lower_forecast, upper_forecast, A),
-            'spl': SPL(
-                A=A,
-                F=upper_forecast,
-                df_train=df_train,
-                quantile=PredictionObject.prediction_interval,
-            )
-            + SPL(
-                A=A,
-                F=lower_forecast,
-                df_train=df_train,
-                quantile=(1 - PredictionObject.prediction_interval),
-            ),
-            'contour': contour(A, F),
-        }
-    ).transpose()
-    per_series.columns = actual.columns
+def smape(actual, forecast, ae):
+    """Accepting abs error already calculated"""
+    return (
+        np.nansum((ae / (abs(forecast) + abs(actual))), axis=0) * 200
+    ) / np.count_nonzero(~np.isnan(actual), axis=0)
 
-    if per_timestamp_errors:
-        smape_df = abs(PredictionObject.forecast - actual) / (
-            abs(PredictionObject.forecast) + abs(actual)
+
+def spl(A, F, quantile, scaler):
+    """Accepting scaler already calculated"""
+    return (
+        np.nanmean(
+            np.where(A >= F, quantile * (A - F), (1 - quantile) * (F - A)), axis=0
         )
-        weight_mean = np.mean(list(series_weights.values()))
-        wsmape_df = (smape_df * series_weights) / weight_mean
-        smape_cons = (np.nansum(wsmape_df, axis=1) * 200) / np.count_nonzero(
-            ~np.isnan(actual), axis=1
-        )
-        per_timestamp = pd.DataFrame({'weighted_smape': smape_cons}).transpose()
-        errors.per_timestamp = per_timestamp
-
-    # this weighting won't work well if entire metrics are NaN
-    # but results should still be comparable
-    errors.avg_metrics_weighted = (per_series * series_weights).sum(
-        axis=1, skipna=True
-    ) / sum(series_weights.values())
-    errors.avg_metrics = per_series.mean(axis=1, skipna=True)
-
-    if full_mae_error:
-        errors.full_mae_errors = abs(A - F)
-
-    errors.per_series_metrics = per_series
-    return errors
+        / scaler
+    )
